
#include <Wire.h>
#include <Adafruit_MPU6050.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_DRV2605.h>

Adafruit_MPU6050 mpu;
Adafruit_DRV2605 drv_step;
Adafruit_DRV2605 drv_freeze;

const float STEP_THRESHOLD = 13.0;
const float PEAK_THRESHOLD = 11.0;          // detect smaller movements for freeze
const unsigned long PEAK_WINDOW = 300;      // time window between fast peaks
const int FREEZE_STEP_COUNT = 5;            // how many fast peaks = freeze

unsigned long lastStepTime = 0;
unsigned long lastPeakTime = 0;
int peakCount = 0;

void setup() {
  Serial.begin(115200);
  Wire.begin();

  if (!mpu.begin()) {
    Serial.println("MPU6050 not detected");
    while (1);
  }

  selectI2CChannel(1);
  if (!drv_step.begin()) {
    Serial.println("DRV2605 (step) not found");
    while (1);
  }
  drv_step.selectLibrary(1);
  drv_step.setMode(DRV2605_MODE_INTTRIG);

  selectI2CChannel(2);
  if (!drv_freeze.begin()) {
    Serial.println("DRV2605 (freeze) not found");
    while (1);
  }
  drv_freeze.selectLibrary(1);
  drv_freeze.setMode(DRV2605_MODE_INTTRIG);

  Serial.println("System ready.");
}

void loop() {
  sensors_event_t a, g, temp;
  mpu.getEvent(&a, &g, &temp);

  float totalAcc = sqrt(
    a.acceleration.x * a.acceleration.x +
    a.acceleration.y * a.acceleration.y +
    a.acceleration.z * a.acceleration.z
  );

  unsigned long now = millis();
  Serial.print("Acc: ");
  Serial.println(totalAcc);

  // STEP DETECTION
  if (totalAcc > STEP_THRESHOLD && (now - lastStepTime > 500)) {
    selectI2CChannel(1);
    triggerSinglePulse(drv_step, 1);
    Serial.println("STEP DETECTED");
    lastStepTime = now;
  }

  // FREEZE-LIKE DETECTION: rapid small-medium peaks
  if (totalAcc > PEAK_THRESHOLD) {
    if (now - lastPeakTime < PEAK_WINDOW) {
      peakCount++;
    } else {
      peakCount = 1;
    }
    lastPeakTime = now;

    if (peakCount >= FREEZE_STEP_COUNT) {
      selectI2CChannel(2);
      triggerMultiplePulses(drv_freeze, 1, 5, 200);
      Serial.println("FREEZE DETECTED (fast small steps)");
      peakCount = 0;
    }
  }
}

void selectI2CChannel(uint8_t channel) {
  Wire.beginTransmission(0x70);
  Wire.write(1 << channel);
  Wire.endTransmission();
}

void triggerSinglePulse(Adafruit_DRV2605 &drv, uint8_t effect) {
  drv.setWaveform(0, effect);
  drv.setWaveform(1, 0);
  drv.go();
}

void triggerMultiplePulses(Adafruit_DRV2605 &drv, uint8_t effect, int count, int interval) {
  for (int i = 0; i < count; i++) {
    triggerSinglePulse(drv, effect);
    delay(interval);
  }
}
